
//This file is part of ssm - Super Slide Me.
//   ssm - Super Slide Me is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



using Gtk;
using GLib;
using Glade;
using System;
using System.IO;
using System.Text;
using System.Timers;
using System.Threading;

using SdlDotNet;
using ICSharpCode.SharpZipLib.Zip;

namespace ssm {

	public class functions {

		// vettore di messaggi
		public static string[] mex_text = new string[41];

		public functions() {}

		// calcola la durata della presentazione
		public static string show_time(ref Gtk.SpinButton spinSec, int n_foto) {
			int s = ((n_foto * Convert.ToInt32(spinSec.Value)) + 5) % 60;
			int m = (((n_foto * Convert.ToInt32(spinSec.Value)) + 5) - s) / 60;
			int mm = m % 60;
			int h = (m - mm) / 60;

			return (h.ToString("D2") + "." + mm.ToString("D2") + "." + s.ToString("D2"));
		}

		// stampa un messaggio m di tipo mt dalla finestra f
		public static void message_box(Gtk.Window f, string m, MessageType mt) {
			MessageDialog msg = new MessageDialog(f,DialogFlags.Modal,mt,ButtonsType.Ok,m);
			msg.Run();
			msg.Destroy();
		}

		// stampa una domanda m dalla finestra f
		// se cancel  true aggiunge anche il tasto cancel
		// ritorna la scelta dell'utente
		public static ResponseType question_box(Gtk.Window f, string m, bool cancel) {
			MessageDialog msg;

			if (!cancel) msg = new MessageDialog(f,DialogFlags.Modal,MessageType.Question,ButtonsType.YesNo,m);
			else {
				msg = new MessageDialog(f,DialogFlags.Modal,MessageType.Question,ButtonsType.None,m);
				msg.AddButton(Gtk.Stock.Cancel,ResponseType.Cancel);
				msg.AddButton(Gtk.Stock.No,ResponseType.No);
				msg.AddButton(Gtk.Stock.Yes,ResponseType.Yes);
			}
			
			ResponseType risposta = (ResponseType)msg.Run();
			msg.Destroy();
			
			return risposta;
		}

		// ritorna un cursore invisibile
		public static Gdk.Cursor ghost_cursor() {
			Gdk.Color col = new Gdk.Color(0,0,0);
			Gdk.Pixmap map = Gdk.Pixmap.CreateBitmapFromData(null,"",1,1);
			Gdk.Cursor cur = new Gdk.Cursor(map,map,col,col,0,0);

			return cur;
		}

		// ridimensione l'immagine per l'anteprima
		public static void size_preview(ref Gdk.Pixbuf pix) {
			try {
				int h = (pix.Height * 150) / pix.Width;
				pix = pix.ScaleSimple(150,h,Gdk.InterpType.Bilinear);
			} catch {}
		}

		// ridimensiona l'immagine per l'anteprima a destra
		public static void size_preview_large(ref Gdk.Pixbuf pix, int wo, int ho, bool force, bool prop, Gdk.InterpType it) {
			try {
				int h = 0, w = 0;
				int nw = wo-250; // larghezza finestra - 200 (dimensione galleria a sx) - 50 (margine)
				int nh = ho-150; // altezza finestra - 100 (margine)
			
				// ridimensiona solo se  pi grande oppure se  forzato
				if ((pix.Width > nw || pix.Height > nh) || (force)) {
					if (prop) {
						// prova a ridimensionare prima per larghezza
						h = (pix.Height * nw) / pix.Width;
				
						if (h > nh) {
							// ridimensiona per altezza
							w = (pix.Width * nh) / pix.Height;
						}
				
						if (w == 0) pix = pix.ScaleSimple(nw,h,it);
						else pix = pix.ScaleSimple(w,nh,it);
					} else {
						// non mantiene le proporzioni
						pix = pix.ScaleSimple(nw,nh,it);
					}
				}
			} catch {}
		}

		// ruota di 90 in senso orario l'immagine pix
		public static void rotate_right(ref Gdk.Pixbuf pix) {
			pix = pix.RotateSimple(Gdk.PixbufRotation.Clockwise);
		}

		// ruota di 90 in senso antiorario l'immagine pix
		public static void rotate_left(ref Gdk.Pixbuf pix) {
			pix = pix.RotateSimple(Gdk.PixbufRotation.Counterclockwise);
		}

		// capovolge l'immagine pix ruotandola di 180
		public static void rotate_down(ref Gdk.Pixbuf pix) {
			pix = pix.RotateSimple(Gdk.PixbufRotation.Upsidedown);
		}

		// carica un vettore di foto nella lista delle anteprime
		public static void load_dir(string[] elenco, Gtk.ListStore store, string[] rot) {
			try {
				int n = ssm.n_foto;

				// scorre tutta la lista e prende in considerazione solo i file corretti
				for (int i = 0; i < elenco.Length; i++) {
					if (!File.Exists(elenco[i])) continue;
				
					// controlla l'estensione
					string ext = elenco[i].Substring(elenco[i].Length-4);

					if (ext.ToLower() == ".jpg" || ext.ToLower() == ".png" || ext.ToLower() == ".gif" || ext.ToLower() == "jpeg") {
						Gdk.Pixbuf pix = new Gdk.Pixbuf(elenco[i]);

						// controlla se c' la rotation
						if (rot != null) {
							if (rot[i] == "l") rotate_left(ref pix);
							else if (rot[i] == "r") rotate_right(ref pix);
							else if (rot[i] == "d") rotate_down(ref pix);
						}
						
						// ridimensiona l'immagine per l'anteprima
						size_preview(ref pix);
					
						if (rot == null) store.AppendValues(pix,elenco[i],"");
						else store.AppendValues(pix,elenco[i],rot[i]);

						n++;
						
						// tiene libera la GUI
						while (Gtk.Application.EventsPending()) Gtk.Application.RunIteration();
					}
				}

				ssm.n_foto = n;
			} catch {
				functions.message_box(null,functions.mex_text[0],MessageType.Error);
			}
		}

		// crea una filechooserdialog
		public static void create_chooser(ref Gtk.FileChooserDialog chooser, string title, Gtk.Window f, Gtk.FileChooserAction action) {
			chooser = new Gtk.FileChooserDialog(title,f,action);

			if (action != Gtk.FileChooserAction.Save) {
				chooser.AddButton(Gtk.Stock.Cancel,Gtk.ResponseType.Cancel);
				chooser.AddButton(Gtk.Stock.Open,Gtk.ResponseType.Ok);
			} else {
				chooser.AddButton(Gtk.Stock.Cancel,Gtk.ResponseType.Cancel);
				chooser.AddButton(Gtk.Stock.Save,Gtk.ResponseType.Ok);
			}
		}

		// crea un filtro personalizzato per la filechooserdialog
		public static void create_filter(ref Gtk.FileFilter filter, string[] ext, string n) {
			string nf = "(";
			for (int i = 0; i < ext.Length; i++) {
				filter.AddPattern(ext[i]);
				
				nf+= ext[i];
				if (i < ext.Length-1) nf+=",";
			}
			
			nf+= ")";
			filter.Name = n + " " + nf;
		}
	}

	public class ssm {

		[Widget] Gtk.Window Form1;
		[Widget] Gtk.Button cmdAdd, cmdAddDir, cmdElimina, cmdPulisci, cmdSfoglia, cmdOrario, cmdAntiorario, cmdCapovolgi, cmdCrea;
		[Widget] Gtk.MenuItem mnuNew, mnuPrimo, mnuSecondo, mnuTerzo, mnuQuarto, mnuQuinto, mnuImpostazioni, mnuAiuto;
		[Widget] Gtk.ImageMenuItem mnuEsci, mnuPreferenze, mnuSalva, mnuSalvaNome, mnuApriP, mnuApriG, mnuChiudiP, mnuCrea, mnuChiudiG, mnuPulisci, mnuAvvia, mnuRecenti, mnuInformazioni, mnuPromuovi;
		[Widget] Gtk.Notebook scheda;
		[Widget] Gtk.SpinButton spinSec, spinComp;
		[Widget] Gtk.Entry txtNome, txtSuono;
		[Widget] Gtk.TreeView lista;
		[Widget] Gtk.Image imgFoto;
		[Widget] Gtk.Statusbar statusBar;
		[Widget] Gtk.RadioButton optMin, optMed, optMax, optOriginali, optJpg, optPng, optBilinear, optHyper;
		[Widget] Gtk.CheckButton chkForce, chkProporzioni;
		[Widget] Gtk.ProgressBar barWork;
		[Widget] Gtk.Label lblSchedaUno, lblSchedaDue, lblAdd, lblAddDir, lblElimina, lblPulisci, lblCrea, lblNome, lblSecondi, lblSuono, lblExpander, lblDimensioni, lblCompressione;

		ListStore store;
		Gtk.FileChooserDialog chooser;
		Gtk.HBox hLeft, hRight, hDown;
		Gtk.Image imgLeft, imgRight, imgDown;
		
		System.Timers.Timer tmr = new System.Timers.Timer(200);
		ThreadNotify notify;
		
		bool activated = false;
		string tip_uno, tip_due, tip_tre;
		
		public static bool unix_sys = false, conf_elimina = false, conf_pulisci = false, apri_big = false, apri_galleria = false, close_window = true, propor = true;
		public static string nome_gallery = "", path = "", suono = "", nome_progetto = "", c_dir, seconds = "", file_to_open = "", p_language;
		public static string[] images_list, recent_list;
		public static int spin_sec = 0, n_foto = 0, filtro = 0;

		public static void Main(string[] argv) {
			try {
				if (argv.Length > 0) file_to_open = argv[0];
			} catch (Exception e) {}
			
			new ssm();
		}
		
		// costruttore
		public ssm() {
			Gtk.Application.Init();
			
			Glade.XML glade = new Glade.XML("ssm.glade","Form1");
			glade.Autoconnect(this);

			store = new ListStore(typeof(Gdk.Pixbuf),typeof(string),typeof(string));
			recent_list = new string[5];
			
			// controlla il sistema operativo
			unix_sys = !(Environment.OSVersion.Platform == PlatformID.Win32S || Environment.OSVersion.Platform == PlatformID.Win32Windows || Environment.OSVersion.Platform == PlatformID.Win32NT);

			// ricava il path dell'eseguibile
			c_dir= System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);
			
			if (!unix_sys) c_dir+="\\";
			else c_dir+="/";

			// segnali
			Form1.MapEvent += Form1_activate;
			Form1.DeleteEvent += Form1_delete;
			Form1.SizeAllocated += Form1_resize;
			mnuNew.Activated += mnuNew_clicked;
			mnuEsci.Activated += mnuEsci_clicked;
			mnuPreferenze.Activated += mnuPreferenze_clicked;
			mnuApriP.Activated += mnuApriP_clicked;
			mnuApriG.Activated += mnuApriG_clicked;
			mnuCrea.Activated += mnuCrea_clicked;
			mnuSalva.Activated += mnuSalva_clicked;
			mnuSalvaNome.Activated += mnuSalvaNome_clicked;
			mnuChiudiP.Activated += mnuChiudiP_clicked;
			mnuChiudiG.Activated += mnuChiudiG_clicked;
			mnuPrimo.Activated += mnuPrimo_clicked;
			mnuSecondo.Activated += mnuSecondo_clicked;
			mnuTerzo.Activated += mnuTerzo_clicked;
			mnuQuarto.Activated += mnuQuarto_clicked;
			mnuQuinto.Activated += mnuQuinto_clicked;
			mnuPulisci.Activated += mnuPulisci_clicked;
			mnuAvvia.Activated += mnuAvvia_clicked;
			mnuInformazioni.Activated += mnuInformazioni_clicked;
			mnuPromuovi.Activated += mnuPromuovi_clicked;
			cmdAdd.Clicked += cmdAdd_clicked;
			cmdAddDir.Clicked += cmdAddDir_clicked;
			cmdElimina.Clicked += cmdElimina_clicked;
			cmdPulisci.Clicked += cmdPulisci_clicked;
			cmdSfoglia.Clicked += cmdSfoglia_clicked;
			cmdCrea.Clicked += cmdCrea_clicked;
			cmdOrario.Clicked += cmdOrario_clicked;
			cmdAntiorario.Clicked += cmdAntiorario_clicked;
			cmdCapovolgi.Clicked += cmdCapovolgi_clicked;
			txtNome.Changed += txtNome_changed;
			spinSec.ValueChanged += spinSec_change;
			tmr.Elapsed += new ElapsedEventHandler(tmr_elapsed);
			lista.Selection.Changed += lista_changed;
			lista.RowActivated += lista_doubleclick;

			notify = new ThreadNotify(new ReadyEvent(move_progress));

			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!unix_sys) icons[0] = new Gdk.Pixbuf(c_dir + "pixmaps\\icon.png");
			else icons[0] = new Gdk.Pixbuf(c_dir + "pixmaps/icon.png");
			Form1.GdkWindow.IconList = icons;
				
			// carica le opzioni
			if (!load_options()) {
				conf_elimina = true;
				conf_pulisci = true;
				apri_big = false;
				apri_galleria = false;
				p_language = "English";
			}

			// carica il linguaggio corretto
			load_language();
			load_messages();

			Tooltips tips = new Tooltips();
			tips.SetTip(cmdAntiorario,tip_uno,"");
			tips.SetTip(cmdOrario,tip_due,"");
			tips.SetTip(cmdCapovolgi,tip_tre,"");
			
			Form1.ShowAll();

			Gtk.Application.Run();
		}

		// apertura della finestra
		public void Form1_activate(object o, MapEventArgs args) {
			try {
				if (activated) return;
				activated = true;

				lista.Model = store;
				lista.AppendColumn("",new CellRendererPixbuf(),"pixbuf",0);
				lista.AppendColumn("",new CellRendererText(),"text",1).Visible = false;
				lista.AppendColumn("",new CellRendererText(),"text",2).Visible = false;

				// sistema le icone dei tasti
				adjust_icons(hRight,imgRight,cmdOrario,c_dir + "pixmaps/rotate_right.png");
				adjust_icons(hLeft,imgLeft,cmdAntiorario,c_dir + "pixmaps/rotate_left.png");
				adjust_icons(hDown,imgDown,cmdCapovolgi,c_dir + "pixmaps/rotate_down.png");

				mnuCrea.Sensitive = false;
				mnuSalva.Sensitive = false;
				mnuSalvaNome.Sensitive = false;
				mnuChiudiP.Sensitive = false;
				mnuChiudiG.Sensitive = false;
				mnuAvvia.Sensitive = false;
				optMed.Active = true;
				optJpg.Active = true;
				optBilinear.Active = true;
				chkForce.Active = false;
				chkProporzioni.Active = true;

				if (apri_big) Form1.Maximize();
			
				// crea i documenti recenti
				mnuPrimo.Visible = false;
				mnuSecondo.Visible = false;
				mnuTerzo.Visible = false;
				mnuQuarto.Visible = false;
				mnuQuinto.Visible = false;
				for (int i = 0; i < 5; i++) recent_list[i] = "";
				load_recent();

				// controlla se  stato passato un file al programma
				if (file_to_open != "") open_recent(file_to_open);
			} catch {}
		}

		// chiusura della finestra
		public void Form1_delete(object o, DeleteEventArgs args) {
			try {
				// chiude il progetto o la galleria
				if (mnuChiudiP.Sensitive) {
					mnuChiudiP.Activate();
					
					// controlla che non sia stato risposto cancel
					if (!close_window) {
						// deve annullare l'evento destroy della finestra
						args.RetVal = true;
						return;
					}
				}
				if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();
			
				// salva le opzioni
				save_options();
			
				// salva i file recenti
				save_recent();
			
				Gtk.Application.Quit();
			} catch {}
		}

		public void mnuInformazioni_clicked(object o, EventArgs args) {
			new informazioni();
		}

		public void mnuPromuovi_clicked(object o, EventArgs args) {
			
		}

		// ridimensionamento della finestra
		public void Form1_resize(object o, EventArgs args) {
			try {
				if (imgFoto.Pixbuf == null) return;
			
				set_image_size();
			} catch {}
		}

		// cambio del valore dei secondi
		public void spinSec_change(object o, EventArgs args) {
			if (n_foto > 0) statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, n_foto));
		}

		// apre lo wizard per la creazione di una galleria fotografica
		public void mnuNew_clicked(object o, EventArgs args) {
			// se c' una galleria aperta la chiude
			if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();

			new wizardOne(store,scheda,spinSec,txtSuono,txtNome,mnuNew,statusBar,mnuSalva,mnuSalvaNome,mnuChiudiP,mnuCrea,Form1,barWork);
		}

		// termina il programma
		public void mnuEsci_clicked(object o, EventArgs args) {
			try {
				// chiude il progetto o la galleria
				if (mnuChiudiP.Sensitive) {
					mnuChiudiP.Activate();

					// controlla che non sia stato risposto cancel
					if (!close_window) return;
				}
				if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();
			
				// salva le opzioni
				save_options();

				// salva i file recenti
				save_recent();
			
				Gtk.Application.Quit();
			} catch {}
		}

		// apre la finestra delle preferenze
		public void mnuPreferenze_clicked(object o, EventArgs args) {
			new preferenze();
		}

		// crea la galleria fotografica
		public void mnuCrea_clicked(object o, EventArgs args) {
			try {
				if (mnuNew.Sensitive) return;
				if (n_foto == 0) {
					// messaggio di warning
					functions.message_box(Form1,functions.mex_text[1],MessageType.Warning);

					return;
				}
				if (txtNome.Text.Trim() == "") {
					// messaggio di warning
					functions.message_box(Form1,functions.mex_text[2],MessageType.Warning);

					return;
				}

				// apre la finestra di dialogo per la scelta del path e del nome del progetto
				functions.create_chooser(ref chooser,functions.mex_text[26],Form1,Gtk.FileChooserAction.Save);

				// crea un filtro per l'estensione progetto
				string[] ext = {"*.ssg"};
				Gtk.FileFilter filter = new Gtk.FileFilter();
				functions.create_filter(ref filter,ext,functions.mex_text[27]);
				chooser.AddFilter(filter);

				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				// controlla la scelta
				if (risp == ResponseType.Ok) {
					// toglie l'estensione se  stata scritta
					string f = chooser.Filename;
					if (f.EndsWith(".ssg")) f = f.Substring(0,f.Length-4);
				
					// controlla se il file esiste gi
					if (File.Exists(f + ".ssg")) {
						// chiede conferma
						ResponseType risposta = functions.question_box(Form1,functions.mex_text[28],false);

						if (risposta == ResponseType.No) return;
					}

					// richiama la funzione per il salvataggio di tutta la galleria
					start_progress();
					save_gallery(f + ".ssg");
					stop_progress();

					// controlla se aprire automaticamente la galleria creata
					if (apri_galleria) {
						if (mnuChiudiP.Sensitive) {
							mnuChiudiP.Activate();

							// controlla che non sia stato risposto cancel
							if (!close_window) return;
						}
						if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();

						start_progress();
						load_gallery(f + ".ssg");
						stop_progress();

						// inserisce il progetto nei recenti
						put_into_recent(f + ".ssg");
					}
				}
			
				chooser.Destroy();
			} catch {}
		}

		// apre un progetto
		public void mnuApriP_clicked(object o, EventArgs args) {
			try {
				// chiude il progetto corrente
				if (mnuChiudiP.Sensitive) {
					mnuChiudiP.Activate();

					// controlla che non sia stato risposto cancel
					if (!close_window) return;
				}
				if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();
			
				functions.create_chooser(ref chooser,functions.mex_text[29],Form1,Gtk.FileChooserAction.Open);
				chooser.SelectMultiple = false;
			
				// crea un filtro per l'estensione progetto
				string[] ext = {"*.ssp"};
				Gtk.FileFilter filter = new Gtk.FileFilter();
				functions.create_filter(ref filter,ext,functions.mex_text[30]);
				chooser.AddFilter(filter);
			
				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				if (risp == ResponseType.Ok) {
					load_project(chooser.Filename);

					// inserisce il progetto nei recenti
					put_into_recent(chooser.Filename);
				}

				chooser.Destroy();
			} catch {}
		}

		public void mnuApriG_clicked(object o, EventArgs args) {
			try {
				// chiude il progetto corrente
				if (mnuChiudiP.Sensitive) {
					mnuChiudiP.Activate();

					// controlla che non sia stato risposto cancel
					if (!close_window) return;
				}
				if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();

				functions.create_chooser(ref chooser,functions.mex_text[31],Form1,Gtk.FileChooserAction.Open);
				chooser.SelectMultiple = false;
			
				// crea un filtro per l'estensione progetto
				string[] ext = {"*.ssg"};
				Gtk.FileFilter filter = new Gtk.FileFilter();
				functions.create_filter(ref filter,ext,functions.mex_text[27]);
				chooser.AddFilter(filter);
			
				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				if (risp == ResponseType.Ok) {
					start_progress();
					load_gallery(chooser.Filename);
					stop_progress();

					// inserisce il progetto nei recenti
					put_into_recent(chooser.Filename);
				}

				chooser.Destroy();
			} catch {}
		}

		// chiude la galleria
		public void mnuChiudiG_clicked(object o, EventArgs args) {
			try {
				nome_gallery = "";
				path = "";
				Form1.Title = "Super Slide Me";
				spin_sec = 0;
				n_foto = 0;
				filtro = 0;
				statusBar.Push(0,functions.mex_text[24] + ": " + n_foto);
				string home;

				// rimuove la cartella tmp nella in home/ssm se esiste
				if (!unix_sys) {
					home = Environment.GetEnvironmentVariable("USERPROFILE") + "\\ssm\\tmp";
				} else {
					home = Environment.GetEnvironmentVariable("HOME") + "/.ssm/tmp";
				}

				if (Directory.Exists(home)) Directory.Delete(home,true);

				mnuChiudiG.Sensitive = false;
				mnuAvvia.Sensitive = false;
			} catch {
				functions.message_box(Form1,functions.mex_text[3],MessageType.Error);
			}
		}

		// chiude il progetto corrente
		public void mnuChiudiP_clicked(object o, EventArgs args) {
			try {
				if (mnuNew.Sensitive) return;

				// se c' gi un altro progetto aperto chiede conferma
				if (!mnuNew.Sensitive) {
					ResponseType risposta = functions.question_box(Form1,functions.mex_text[32],true);

					if (risposta == ResponseType.Yes) mnuSalva.Activate();
					else if (risposta == ResponseType.Cancel) {
						close_window = false;
						return;
					}
				}

				close_window = true;
			
				store.Clear();
				n_foto = 0;
				statusBar.Push(0,functions.mex_text[24] + ": " + n_foto);
				imgFoto.Pixbuf = null;
				txtNome.Text = ""; spinSec.Value = 4; txtSuono.Text = ""; spinComp.Value = 5; 
				nome_gallery = ""; path = ""; suono = ""; nome_progetto = ""; 

				mnuChiudiP.Sensitive = false;
				mnuSalva.Sensitive = false;
				mnuSalvaNome.Sensitive = false;
				mnuNew.Sensitive = true;
				mnuCrea.Sensitive = false;
				optMed.Active = true;
				optJpg.Active = true;
				optBilinear.Active = true;
				scheda.Sensitive = false;
				chkForce.Active = false;
				chkProporzioni.Active = true;
				Form1.Title = "Super Slide Me";
			} catch {
				functions.message_box(Form1,functions.mex_text[4],MessageType.Error);
			}
		}

		// salva progetto
		public void mnuSalva_clicked(object o, EventArgs args) {
			try {
				// controlla se  presente almeno una foto
				if (n_foto == 0) {
					// messaggio di warning
					functions.message_box(Form1,functions.mex_text[1],MessageType.Warning);

					return;
				}
			
				// se non si tratta di un progetto gi esistente richiama la funzione salva con nome
				if (nome_progetto == "") mnuSalvaNome.Activate();
				else save_project(nome_progetto);
			} catch {}
		}

		public void mnuSalvaNome_clicked(object o, EventArgs args) {
			try {
				// controlla se  presente almeno una foto
				if (n_foto == 0) {
					// messaggio di warning
					functions.message_box(Form1,functions.mex_text[1],MessageType.Warning);

					return;
				}

				// apre la finestra di dialogo per la scelta del path e del nome del progetto
				functions.create_chooser(ref chooser,functions.mex_text[33],Form1,Gtk.FileChooserAction.Save);

				// crea un filtro per l'estensione progetto
				string[] ext = {"*.ssp"};
				Gtk.FileFilter filter = new Gtk.FileFilter();
				functions.create_filter(ref filter,ext,functions.mex_text[30]);
				chooser.AddFilter(filter);

				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				// controlla la scelta
				if (risp == ResponseType.Ok) {
					// toglie l'estensione se  stata scritta
					string f = chooser.Filename;
					if (f.EndsWith(".ssp")) f = f.Substring(0,f.Length-4);
					
					// controlla se il file esiste gi
					if (File.Exists(f + ".ssp")) {
						// chiede conferma
						ResponseType risposta = functions.question_box(Form1,functions.mex_text[28],false);

						if (risposta == ResponseType.No) return;
					}

					// richiama la funzione per il salvataggio del progetto
					save_project(f + ".ssp");
					nome_progetto = f + ".ssp";
					put_into_recent(f + ".ssp");
				}
			
				chooser.Destroy();
			} catch {}
		}

		public void mnuPrimo_clicked(object o, EventArgs args) {
			open_recent(recent_list[0]);	
		}

		public void mnuSecondo_clicked(object o, EventArgs args) {
			open_recent(recent_list[1]);	
		}
		
		public void mnuTerzo_clicked(object o, EventArgs args) {
			open_recent(recent_list[2]);	
		}
		
		public void mnuQuarto_clicked(object o, EventArgs args) {
			open_recent(recent_list[3]);	
		}
		
		public void mnuQuinto_clicked(object o, EventArgs args) {
			open_recent(recent_list[4]);	
		}

		// elimina tutti i documenti recenti
		public void mnuPulisci_clicked(object o, EventArgs args) {
			try {
				for (int i = 0; i < 5; i++) recent_list[i] = "";

				((Gtk.Label)mnuPrimo.Child).Text = "";
				((Gtk.Label)mnuSecondo.Child).Text = "";
				((Gtk.Label)mnuTerzo.Child).Text = "";
				((Gtk.Label)mnuQuarto.Child).Text = "";
				((Gtk.Label)mnuQuinto.Child).Text = "";

				mnuPrimo.Visible = false;
				mnuSecondo.Visible = false;
				mnuTerzo.Visible = false;
				mnuQuarto.Visible = false;
				mnuQuinto.Visible = false;
			} catch {
				functions.message_box(Form1,functions.mex_text[5],MessageType.Error);
			}
		}

		// avvia la presentazione
		public void mnuAvvia_clicked(object o, EventArgs args) {
			if (path != "") new presentazione(path);
		}

		// modifica del nome della galleria
		public void txtNome_changed(object o, EventArgs args) {
			Form1.Title = "Super Slide Me - " + txtNome.Text;
		}
		
		// aggiunge altre foto alla galleria
		public void cmdAdd_clicked(object o, EventArgs args) {
			try {
				functions.create_chooser(ref chooser,functions.mex_text[34],Form1,Gtk.FileChooserAction.Open);
			
				chooser.SelectMultiple = true;
			
				// crea un filtro per le immagini
				string[] ext = {"*.jpg","*.jpeg","*.gif","*.png","*.JPG","*.JPEG","*.GIF","*.PNG"};
				Gtk.FileFilter filter = new Gtk.FileFilter();
				functions.create_filter(ref filter,ext,functions.mex_text[35]);
				chooser.AddFilter(filter);

				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				// controlla la scelta
				if (risp == ResponseType.Ok) {
					start_progress();
					functions.load_dir(chooser.Filenames,store,null);
					stop_progress();
					statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, n_foto));
				}

				chooser.Destroy();
			} catch {}
		}

		// aggiunge altre directory alla galleria
		public void cmdAddDir_clicked(object o, EventArgs args) {
			try {
				functions.create_chooser(ref chooser,functions.mex_text[36],Form1,Gtk.FileChooserAction.SelectFolder);

				chooser.SelectMultiple = true;

				Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
				chooser.Hide();

				// controlla la scelta
				if (risp == ResponseType.Ok) {
					int n = 0, c = 0;
					string[] d = chooser.Filenames;
				
					// calcola il numero di file per creare l'array
					for (int i = 0; i < d.Length; i++) n+= Directory.GetFiles(d[i]).Length;
				
					// prende tutti i file presenti nelle directory selezionate
					string[] f = new string[n];
				
					for (int i = 0; i < d.Length; i++) {
						string[] ft = Directory.GetFiles(d[i]);
						for (int j = 0; j < ft.Length; j++) {
							f[c] = ft[j];
							c++;
						}
					}

					start_progress();
					functions.load_dir(f,store,null);
					stop_progress();
					statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, n_foto));
				}
			} catch {}
		}

		// elimina la foto selezionata
		public void cmdElimina_clicked(object o, EventArgs args) {
			try {
				// controlla se c' una foto selezionata
				TreeModel model;
				TreeIter iter;
				ResponseType risposta = ResponseType.No;

				if (!lista.Selection.GetSelected(out model, out iter)) return;
			
				// controlla se bisogna richiedere conferma oppure no
				if (conf_elimina) risposta = functions.question_box(Form1,functions.mex_text[37],false);

				// controlla la risposta
				if (risposta == ResponseType.Yes || !conf_elimina) {
					// elimina la foto selezionata
					store.Remove(ref iter);
					imgFoto.Pixbuf = null;
					n_foto--;
					if (n_foto > 0) statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, n_foto));
					else statusBar.Push(0,functions.mex_text[24] + ": " + n_foto);
				}
			} catch {
				functions.message_box(Form1,functions.mex_text[6],MessageType.Error);
			}
		}

		// svuota tutta la galleria
		public void cmdPulisci_clicked(object o, EventArgs args) {
			try {
				// controlla se la galleria contiene qualche immagine
				TreeIter iter;
				ResponseType risposta = ResponseType.No;

				if (!store.GetIterFirst(out iter)) return;

				// controlla se bisogna richiedere conferma oppure no
				if (conf_pulisci) risposta = functions.question_box(Form1,functions.mex_text[38],false);

				// controlla la risposta
				if (risposta == ResponseType.Yes || !conf_pulisci) {
					// pulisce tutta la galleria
					store.Clear();
					imgFoto.Pixbuf = null;
					n_foto = 0;
					statusBar.Push(0,functions.mex_text[24] + ": " + n_foto);
				}
			} catch {
				functions.message_box(Form1,functions.mex_text[7],MessageType.Error);
			}
		}

		// apre la selezione file per il suono
		public void cmdSfoglia_clicked(object o, EventArgs args) {
			// apre la finestra di selezione files
			functions.create_chooser(ref chooser,functions.mex_text[39],Form1,Gtk.FileChooserAction.Open);
			chooser.SelectMultiple = false;

			// crea un filtro per i suoni
			string[] ext = {"*.mp3","*.wav","*.ogg"};
			Gtk.FileFilter filter = new Gtk.FileFilter();
			functions.create_filter(ref filter,ext,functions.mex_text[40]);
			chooser.AddFilter(filter);
			
			Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
			chooser.Hide();

			// controlla la scelta
			if (risp == ResponseType.Ok) txtSuono.Text = chooser.Filename;

			chooser.Destroy();
		}

		// crea la galleria fotografica se possibile
		public void cmdCrea_clicked(object o, EventArgs args) {
			mnuCrea.Activate();
		}

		// ruota di 90 in senso orario l'immagine
		public void cmdOrario_clicked(object o, EventArgs args) {
			try {
				TreeModel model;
				TreeIter iter;

				if (!lista.Selection.GetSelected(out model, out iter)) return;
			
				Gdk.Pixbuf pix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString());
		
				functions.rotate_right(ref pix);
			
				// controlla dove deve essere girata la foto originale
				string lato = store.GetValue(iter,2).ToString();
				if (lato == "") store.SetValue(iter,2,"r");
				else if (lato == "l") store.SetValue(iter,2,"");
				else if (lato == "r") store.SetValue(iter,2,"d");
				else if (lato == "d") store.SetValue(iter,2,"l");
			
				imgFoto.FromPixbuf = pix;

				// sistema anche la preview a sinistra
				Gdk.Pixbuf minipix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString()); 
				if (lato == "") functions.rotate_right(ref minipix);
				else if (lato == "r") functions.rotate_down(ref minipix);
				else if (lato == "d") functions.rotate_left(ref minipix);
				functions.size_preview(ref minipix);
				store.SetValue(iter,0,minipix);
			} catch {}
		}

		// ruota di 90 in senso antiorario l'immagine
		public void cmdAntiorario_clicked(object o, EventArgs args) {
			try {
				TreeModel model;
				TreeIter iter;

				if (!lista.Selection.GetSelected(out model, out iter)) return;
			
				Gdk.Pixbuf pix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString());
		
				functions.rotate_left(ref pix);
			
				// controlla dove deve essere girata la foto originale
				string lato = store.GetValue(iter,2).ToString();
				if (lato == "") store.SetValue(iter,2,"l");
				else if (lato == "l") store.SetValue(iter,2,"d");
				else if (lato == "r") store.SetValue(iter,2,"");
				else if (lato == "d") store.SetValue(iter,2,"r");
			
				imgFoto.FromPixbuf = pix;
			
				// sistema anche la preview a sinistra
				Gdk.Pixbuf minipix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString()); 
				if (lato == "") functions.rotate_left(ref minipix);
				else if (lato == "l") functions.rotate_down(ref minipix);
				else if (lato == "d") functions.rotate_right(ref minipix);
				functions.size_preview(ref minipix);
				store.SetValue(iter,0,minipix);
			} catch {}
		}

		// ruota di 180 l'immagine capovolgendola
		public void cmdCapovolgi_clicked(object o, EventArgs args) {
			try {
				TreeModel model;
				TreeIter iter;

				if (!lista.Selection.GetSelected(out model, out iter)) return;
			
				Gdk.Pixbuf pix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString());
		
				functions.rotate_down(ref pix);
			
				// controlla dove deve essere girata la foto originale
				string lato = store.GetValue(iter,2).ToString();
				if (lato == "") store.SetValue(iter,2,"d");
				else if (lato == "l") store.SetValue(iter,2,"r");
				else if (lato == "r") store.SetValue(iter,2,"l");
				else if (lato == "d") store.SetValue(iter,2,"");
			
				imgFoto.FromPixbuf = pix;
			
				// sistema anche la preview a sinistra
				Gdk.Pixbuf minipix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString()); 
				if (lato == "") functions.rotate_down(ref minipix);
				else if (lato == "l") functions.rotate_right(ref minipix);
				else if (lato == "r") functions.rotate_left(ref minipix);
				functions.size_preview(ref minipix);
				store.SetValue(iter,0,minipix);
			} catch {}
		}
		
		// visualizza la foto a destra
		public void lista_changed(object o, EventArgs args) {
			set_image_size();
		}

		// apre la foto nel visualizzatore
		public void lista_doubleclick(object o, RowActivatedArgs args) {
			TreeModel model;
			TreeIter iter;

			lista.Selection.GetSelected(out model, out iter);
			new visualizzatore(store.GetValue(iter,1).ToString(),store.GetValue(iter,2).ToString());
		}

		// sistema l'immagine per l'anteprima a destra
		private void set_image_size() {
			try {
				TreeModel model;
				TreeIter iter;
				int w,h;

				if (!lista.Selection.GetSelected(out model, out iter)) {
					imgFoto.FromPixbuf = null;
				
					return;
				}

				Gdk.Pixbuf pix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString());
			
				Form1.GetSize(out w, out h);
			
				// controlla se l'immagine deve essere girata
				if (store.GetValue(iter,2).ToString() == "l") functions.rotate_left(ref pix);
				else if (store.GetValue(iter,2).ToString() == "r") functions.rotate_right(ref pix);
				else if (store.GetValue(iter,2).ToString() == "d") functions.rotate_down(ref pix);
			
				functions.size_preview_large(ref pix,w,h,false,true,Gdk.InterpType.Bilinear);

				imgFoto.FromPixbuf = pix;
			} catch {}
		}

		// carica le opzioni
		// ritorna true se tutto  ok, false in caso di errore
		private bool load_options() {
			try {
				FileStream fs;
				string home, linea;
			
				if (!unix_sys) {
					// windows
					home = Environment.GetEnvironmentVariable("USERPROFILE");	

					if (!File.Exists(home + "\\ssm\\options")) return false;
					fs = new FileStream(home + "\\ssm\\options",System.IO.FileMode.Open,System.IO.FileAccess.Read);
				} else {
					// unix
					home = Environment.GetEnvironmentVariable("HOME");
				
					if (!File.Exists(home + "/.ssm/options")) return false;
					fs = new FileStream(home + "/.ssm/options",System.IO.FileMode.Open,System.IO.FileAccess.Read);
				}

				StreamReader r = new StreamReader(fs);

				if (r.Peek() > -1) linea = r.ReadLine().Trim();
				else return false;
				if (Convert.ToInt32(linea) != 0) conf_elimina = true;
				else conf_elimina = false;
			
				if (r.Peek() > -1) linea = r.ReadLine().Trim();
				else return false;
				if (Convert.ToInt32(linea) != 0) conf_pulisci = true;
				else conf_pulisci = false;

				if (r.Peek() > -1) linea = r.ReadLine().Trim();
				else return false;
				if (Convert.ToInt32(linea) != 0) apri_big = true;
				else apri_big = false;
			
				if (r.Peek() > -1) linea = r.ReadLine().Trim();
				else return false;
				if (Convert.ToInt32(linea) != 0) apri_galleria = true;
				else apri_galleria = false;

				p_language = r.ReadLine().Trim();
			
				r.Close();
				fs.Close();

				return true;
			} catch {
				functions.message_box(Form1,functions.mex_text[8],MessageType.Error);
				
				return false;
			}
		}

		// salva le opzioni su file di testo
		private void save_options() {
			try {
				FileStream fs;
				string home;
			
				if (!unix_sys) {
					// windows
					home = Environment.GetEnvironmentVariable("USERPROFILE");	
				
					if (!Directory.Exists(home + "\\ssm")) Directory.CreateDirectory(home + "\\ssm");
					fs = new FileStream(home + "\\ssm\\options",System.IO.FileMode.Create,System.IO.FileAccess.Write);
				} else {
					// unix
					home = Environment.GetEnvironmentVariable("HOME");
				
					if (!Directory.Exists(home + "/.ssm")) Directory.CreateDirectory(home + "/.ssm");
					fs = new FileStream(home + "/.ssm/options",System.IO.FileMode.Create,System.IO.FileAccess.Write);
				}

				StreamWriter w = new StreamWriter(fs);
			
				if (conf_elimina) w.WriteLine("1");
				else w.WriteLine("0");

				if (conf_pulisci) w.WriteLine("1");
				else w.WriteLine("0");

				if (apri_big) w.WriteLine("1");
				else w.WriteLine("0");

				if (apri_galleria) w.WriteLine("1");
				else w.WriteLine("0");

				w.WriteLine(p_language);
			
				w.Close();
				fs.Close();
			} catch {
				functions.message_box(Form1,functions.mex_text[9],MessageType.Error);
			}
		}

		// carica il progetto f
		private void load_project(string f) {
			try {
				FileStream fs = new FileStream(f,System.IO.FileMode.Open,System.IO.FileAccess.Read);
				StreamReader r = new StreamReader(fs);

				// legge prima le opzioni
				txtNome.Text = r.ReadLine();
				Form1.Title = "Super Slide Me - " + txtNome.Text;
				txtSuono.Text = r.ReadLine();
				spinSec.Value = Convert.ToInt32(r.ReadLine());
				int q = Convert.ToInt32(r.ReadLine());
				if (q == 0) optMin.Active = true;
				else if (q == 1) optMed.Active = true;
				else if (q == 2) optMax.Active = true;
				else if (q == 3) optOriginali.Active = true;
				q = Convert.ToInt32(r.ReadLine());
				if (q == 0) optJpg.Active = true;
				else optPng.Active = true;
				q = Convert.ToInt32(r.ReadLine());
				if (q == 0) optBilinear.Active = true;
				else optHyper.Active = true;
				chkForce.Active = Convert.ToBoolean(r.ReadLine());
				chkProporzioni.Active = Convert.ToBoolean(r.ReadLine());
				spinComp.Value = Convert.ToInt32(r.ReadLine());
				int n = Convert.ToInt32(r.ReadLine());

				string[] p = new string[n]; // vettore di path
				string[] rot = new string[n]; // vettore di rotazioni
				int i = 0;
			
				// carica tutte le foto
				while (r.Peek() > -1) {
					string linea = r.ReadLine();
					p[i] = linea.Split(';')[0];
					rot[i] = linea.Split(';')[1];
					i++;
				}

				r.Close();
				fs.Close();

				scheda.Sensitive = true;
				nome_progetto = f;
				mnuNew.Sensitive = false;
				mnuCrea.Sensitive = true;
				mnuSalva.Sensitive = true;
				mnuSalvaNome.Sensitive = true;
				mnuChiudiP.Sensitive = true;
			
				// richiama la funzione per caricare tutte le immagini
				start_progress();
				functions.load_dir(p,store,rot);
				stop_progress();
				statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, n_foto));
			} catch {
				functions.message_box(Form1,functions.mex_text[10],MessageType.Error);
			}
		}

		// carica la galleria per la visualizzazione
		private void load_gallery(string f) {
			try {
				FileStream fs;
				ZipInputStream s = new ZipInputStream(File.OpenRead(f));
				ZipEntry entry;
				string home;

				int size = 2048;
				byte[] data = new byte[2048];
			
				entry = s.GetNextEntry();
				s.Read(data,0,size);
				nome_gallery = new UTF8Encoding().GetString(data, 0, size).Split('\n')[0];
				spin_sec = Convert.ToInt32(new ASCIIEncoding().GetString(data, 0, size).Split('\n')[1]);
				propor = Convert.ToBoolean(new ASCIIEncoding().GetString(data, 0, size).Split('\n')[2]);
				filtro = Convert.ToInt32(new ASCIIEncoding().GetString(data, 0, size).Split('\n')[3]);

				Form1.Title = "Super Slide Me - " + nome_gallery;
			
				// salta le due cartelle pixmaps e sound
				for (int i = 0; i < 2; i++) entry = s.GetNextEntry();

				if (!unix_sys) {
					// windows
					home = Environment.GetEnvironmentVariable("USERPROFILE") + "\\ssm\\";
					if (!Directory.Exists(home)) Directory.CreateDirectory(home);
					if (!Directory.Exists(home + "tmp")) Directory.CreateDirectory(home + "tmp");
					home += "tmp\\";
				} else {
					// unix
					home = Environment.GetEnvironmentVariable("HOME") + "/.ssm/";
					if (!Directory.Exists(home)) Directory.CreateDirectory(home);
					if (!Directory.Exists(home + "tmp")) Directory.CreateDirectory(home + "tmp");
					home += "tmp/";
				}

				n_foto = 0;
			
				while ((entry = s.GetNextEntry()) != null) {
					fs = new FileStream(home + entry.Name.Split('/')[1],System.IO.FileMode.Create,System.IO.FileAccess.Write);
					size = 2048;
					while (size > 0) {
						size = s.Read(data,0,data.Length);
						fs.Write(data,0,size);
					}

					fs.Close();

					if (!entry.Name.Split('/')[1].EndsWith("mp3") && !entry.Name.Split('/')[1].EndsWith("wav") && !entry.Name.Split('/')[1].EndsWith("ogg")) n_foto++;
				
					// tiene libera la GUI
					while (Gtk.Application.EventsPending()) Gtk.Application.RunIteration();
				}

				s.Close();

				// richiama il visualizzatore di gallerie
				path = home;

				mnuAvvia.Sensitive = true;
				mnuChiudiG.Sensitive = true;
				Gtk.SpinButton spin = new Gtk.SpinButton(0,30,1);
				spin.Value = spin_sec;
				statusBar.Push(0,functions.mex_text[24] + ": " + n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spin, n_foto));

				mnuAvvia.Activate();
			} catch {
				functions.message_box(Form1,functions.mex_text[11],MessageType.Error);
			}
		}

		// salva il progetto su file f
		private void save_project(string f) {
			try {
				FileStream fs = new FileStream(f,System.IO.FileMode.Create,System.IO.FileAccess.Write);
				StreamWriter w = new StreamWriter(fs);

				// scrive le opzioni
				w.WriteLine(txtNome.Text);
				w.WriteLine(txtSuono.Text);
				w.WriteLine(spinSec.Value);
				if (optMin.Active) w.WriteLine("0");
				else if (optMed.Active) w.WriteLine("1");
				else if (optMax.Active) w.WriteLine("2");
				else if (optOriginali.Active) w.WriteLine("3");
				if (optJpg.Active) w.WriteLine("0");
				else w.WriteLine("1");
				if (optBilinear.Active) w.WriteLine("0");
				else w.WriteLine("1");
				w.WriteLine(chkForce.Active);
				w.WriteLine(chkProporzioni.Active);
				w.WriteLine(spinComp.Value);
				w.WriteLine(n_foto);

				// memorizza il path e la rotazione delle immagini
				TreeIter iter;
				store.GetIterFirst(out iter);
				do {
					w.WriteLine(store.GetValue(iter,1).ToString() + ";" + store.GetValue(iter,2).ToString());
				} while (store.IterNext(ref iter));

				w.Close();
				fs.Close();
			} catch {
				functions.message_box(Form1,functions.mex_text[12],MessageType.Error);
			}
		}

		// salva la galleria su file f
		private void save_gallery(string f) {
			try {
				string home;
				
				if (!unix_sys) {
					// windows
					home = Environment.GetEnvironmentVariable("USERPROFILE") + "\\ssm\\";
					if (!Directory.Exists(home)) Directory.CreateDirectory(home);
					if (!Directory.Exists(home + "tmp")) Directory.CreateDirectory(home + "tmp");
					home += "tmp\\";
				} else {
					// unix
					home = Environment.GetEnvironmentVariable("HOME") + "/.ssm/";
					if (!Directory.Exists(home)) Directory.CreateDirectory(home);
					if (!Directory.Exists(home + "tmp")) Directory.CreateDirectory(home + "tmp");
					home += "tmp/";
				}
			
				// scrive un file contenente alcune informazioni
				FileStream fs = new FileStream(home + "data",System.IO.FileMode.Create,System.IO.FileAccess.Write);
      		StreamWriter w = new StreamWriter(fs);
				
				w.WriteLine(txtNome.Text);
				w.WriteLine(spinSec.Value);
				w.WriteLine(chkProporzioni.Active);
				if (optBilinear.Active) w.WriteLine("0");
				else w.WriteLine("1");
			
				w.Close();
				fs.Close();

				// crea un archivio zip
				ZipOutputStream outStream = new ZipOutputStream(File.Create(f));

				// setta il livello di compressione
				outStream.SetLevel(Convert.ToInt32(spinComp.Value));
			
				// inserisce il file data
				add_to_zip(ref outStream,home + "data","data");

				// rimuove il file temporaneo creato
				File.Delete(home + "data");

				// crea le directory pixmaps e sound
				add_to_zip(ref outStream,"","pixmaps/");
				add_to_zip(ref outStream,"","sound/");
			
				// inserisce il suono se esiste
				if (txtSuono.Text != "" && File.Exists(txtSuono.Text)) {
					string ext = txtSuono.Text.Substring(txtSuono.Text.Length-3);
					if (ext == "mp3" || ext == "ogg" || ext == "wav") add_to_zip(ref outStream,txtSuono.Text,"sound/sound." + ext);
				}

				// copia tutte le foto della galleria nella cartella pixmaps	
				int i = 0;
				TreeIter iter;
				store.GetIterFirst(out iter);
				do {
					Gdk.Pixbuf pix = new Gdk.Pixbuf(store.GetValue(iter,1).ToString());

					// controlla se eseguire una rotation
					if (store.GetValue(iter,2).ToString() == "l") functions.rotate_left(ref pix);
					else if (store.GetValue(iter,2).ToString() == "r") functions.rotate_right(ref pix);
					else if (store.GetValue(iter,2).ToString() == "d") functions.rotate_down(ref pix);

					// ridimensiona in base alla qualit scelta nelle opzioni
					int h = 0, wd = 0;
					if (optMin.Active) { wd = 890; h = 630; } // 640x480
					else if (optMed.Active) { wd = 1050; h = 750; } // 800x600
					else if (optMax.Active) { wd = 1274; h = 918; } // 1024x768

					// se sono state scelte le dimensioni originali non effettua il ridimensionamento
					if (!optOriginali.Active) {
						if (optBilinear.Active) functions.size_preview_large(ref pix,wd,h,chkForce.Active,chkProporzioni.Active,Gdk.InterpType.Bilinear);
						else functions.size_preview_large(ref pix,wd,h,chkForce.Active,chkProporzioni.Active,Gdk.InterpType.Hyper);
					}
					
					// salva l'immagine su file
					string num = i.ToString();
					if (num.Length == 1) num = "00" + num;
					else if (num.Length == 2) num = "0" + num;
					if (optJpg.Active) pix.Save(home + "img" + num + ".jpg","jpeg");
					else pix.Save(home + "img" + num + ".png","png");
					i++;
					
					if (optJpg.Active) {
						add_to_zip(ref outStream,home + "img" + num + ".jpg","pixmaps/img" + num + ".jpg");

						// rimuove la foto temporanea
						File.Delete(home + "img" + num + ".jpg");
					} else {
						add_to_zip(ref outStream,home + "img" + num + ".png","pixmaps/img" + num + ".png");

						// rimuove la foto temporanea
						File.Delete(home + "img" + num + ".png");
					}
				
					// tiene libera la GUI
					while (Gtk.Application.EventsPending()) Gtk.Application.RunIteration();
				} while (store.IterNext(ref iter));

				// elimina la directory temporanea
				Directory.Delete(home,true);

				outStream.Close();
			} catch {
				functions.message_box(Form1,functions.mex_text[13],MessageType.Error);
			}
		}

		// aggiunge un file o una cartella all'archivio outStream	
		private void add_to_zip(ref ZipOutputStream outStream, string f, string n) {
			try {
				FileStream fs;
				byte[] buffer = new byte[0];
				if (f != "") {
					fs = File.OpenRead(f);
					buffer = new byte[fs.Length];
					fs.Read(buffer,0,buffer.Length);
				}
				ZipEntry entry = new ZipEntry(n);
				outStream.PutNextEntry(entry);
				if (f != "") outStream.Write(buffer,0,buffer.Length);
			} catch {
				functions.message_box(Form1,functions.mex_text[14],MessageType.Error);
			}
		}

		// aggiunge le icone ai tasti
		private void adjust_icons(Gtk.HBox h, Gtk.Image i, Gtk.Button b, string f) {
			try {
				h = new HBox(false,0);
				i = new Gtk.Image(f);
				h.PackStart(i,false,false,0);
				i.Show();
				h.Show();
				b.Add(h);
			} catch {}
		}

		// legge da file di testo gli ultimi documenti recenti e li carica nel menu e in un vettore
		private void load_recent() {
			try {
				string home, c;
			
				if (!unix_sys) {
					home = Environment.GetEnvironmentVariable("USERPROFILE") + "\\ssm\\recent";
					if (!File.Exists(home)) return;
					c = "\\";
				} else {
					home = Environment.GetEnvironmentVariable("HOME") + "/.ssm/recent";
					if (!File.Exists(home)) return;
					c = "/";
				}

				FileStream fs = new FileStream(home,System.IO.FileMode.Open,System.IO.FileAccess.Read);
				StreamReader r = new StreamReader(fs);

				for (int i = 0; i < 5 && r.Peek() > -1; i++) {
					string linea = r.ReadLine().Trim();
				
					if (linea == "") return;

					recent_list[i] = linea;
					string file = linea.Substring(linea.LastIndexOf(c)+1);

					switch (i) {
					case 0: 
						((Gtk.Label)mnuPrimo.Child).Text = file;
						mnuPrimo.Visible = true;
						break;
					case 1:
						((Gtk.Label)mnuSecondo.Child).Text = file;
						mnuSecondo.Visible = true;
						break;
					case 2:
						((Gtk.Label)mnuTerzo.Child).Text = file;
						mnuTerzo.Visible = true;
						break;
					case 3:
						((Gtk.Label)mnuQuarto.Child).Text = file;
						mnuQuarto.Visible = true;
						break;
					case 4:
						((Gtk.Label)mnuQuinto.Child).Text = file;
						mnuQuinto.Visible = true;
						break;
					}	
				}

				r.Close();
				fs.Close();
			} catch {
				functions.message_box(Form1,functions.mex_text[15],MessageType.Error);
			}
		}

		// salva la lista dei recenti dal vettore al file recent
		private void save_recent() {
			try {
				string home;
			
				if (!unix_sys) {
					home = Environment.GetEnvironmentVariable("USERPROFILE") + "\\ssm\\recent";
				} else {
					home = Environment.GetEnvironmentVariable("HOME") + "/.ssm/recent";
				}

				FileStream fs = new FileStream(home,System.IO.FileMode.Create,System.IO.FileAccess.Write);
				StreamWriter w = new StreamWriter(fs);
			
				for (int i = 0; i < 5; i++) {
					if (recent_list[i] != "") w.WriteLine(recent_list[i]);
				}

				w.Close();
				fs.Close();
			} catch {
				functions.message_box(Form1,functions.mex_text[16],MessageType.Error);
			}
		}

		// inserisce in testa ai recenti il file f
		private void put_into_recent(string f) {
			try {
				for (int i = 3; i > -1; i--) {
					if (recent_list[i] != "") {
						recent_list[i+1] = recent_list[i];

						switch (i) {
						case 0:
							((Gtk.Label)mnuSecondo.Child).Text = ((Gtk.Label)mnuPrimo.Child).Text;
							mnuSecondo.Visible = true;
							break;
						case 1:
							((Gtk.Label)mnuTerzo.Child).Text = ((Gtk.Label)mnuSecondo.Child).Text;
							mnuTerzo.Visible = true;
							break;
						case 2:
							((Gtk.Label)mnuQuarto.Child).Text = ((Gtk.Label)mnuTerzo.Child).Text;
							mnuQuarto.Visible = true;
							break;
						case 3:
							((Gtk.Label)mnuQuinto.Child).Text = ((Gtk.Label)mnuQuarto.Child).Text;
							mnuQuinto.Visible = true;
							break;
						}
					}
				}
			
				string c;
				if (!unix_sys) c = "\\";
				else c = "/";

				((Gtk.Label)mnuPrimo.Child).Text = f.Substring(f.LastIndexOf(c)+1);
				mnuPrimo.Visible = true;
				recent_list[0] = f;
			} catch {}
		}

		// apre il file recente f se possibile
		private void open_recent(string f) {
			try {
				// controlla se il file esiste
				if (!File.Exists(f)) {
					// messaggio di errore
					functions.message_box(Form1,functions.mex_text[17],MessageType.Error);

					return;
				}

				// controlla che estensione ha per sapere se  un progetto o una galleria
				if (f.Substring(f.Length - 3) == "ssp") {
					if (mnuChiudiP.Sensitive) {
						mnuChiudiP.Activate();

						// controlla che non sia stato risposto cancel
						if (!close_window) return;
					}
					if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();
				
					load_project(f);

					// inserisce il progetto nei recenti
					put_into_recent(f);
				} else if (f.Substring(f.Length - 3) == "ssg") {
					if (mnuChiudiP.Sensitive) {
						mnuChiudiP.Activate();

						// controlla che non sia stato risposto cancel
						if (!close_window) return;
					}
					if (mnuChiudiG.Sensitive) mnuChiudiG.Activate();
				
					start_progress();
					load_gallery(f);
					stop_progress();

					// inserisce il progetto nei recenti
					put_into_recent(f);
				} else {
					// messaggio di errore
					functions.message_box(Form1,functions.mex_text[18],MessageType.Error);
				}
			} catch {
				functions.message_box(Form1,functions.mex_text[19],MessageType.Error);
			}
		}
		
		private void start_progress() {
			tmr.Start();
		}

		private void stop_progress() {
			try {
				tmr.Stop();
				barWork.Fraction = 0;
			} catch {}
		}

		public void tmr_elapsed(object o, ElapsedEventArgs args) {
			notify.WakeupMain();
		}

		private void move_progress() {
			try {
				barWork.Pulse();
			} catch {}
		}

		// setta tutti i messaggi a seconda della lingua scelta
		public void load_messages() {
			try {
				FileStream fs;
				string path = c_dir + "lang/";

				if (p_language == "English") path += "en";
				else if (p_language == "Italiano") path += "it";
				else if (p_language == "Franais") path += "fr";
				else if (p_language == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);

				// si posiziona nella sezione dei messaggi
				for (int i = 0; i < 5; i++) while (r.Peek() > -1 && r.ReadLine() != "***");
			
				// carica il vettore dei messaggi
				for (int i = 0; r.Peek() > -1; i++) functions.mex_text[i] = r.ReadLine();

				r.Close();
				fs.Close();
			} catch {}
		}

		// setta tutti gli oggetti a seconda della lingua scelta
		public void load_language() {
			try {
				FileStream fs;
				string path = c_dir + "lang/";

				if (p_language == "English") path += "en";
				else if (p_language == "Italiano") path += "it";
				else if (p_language == "Franais") path += "fr";
				else if (p_language == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);
				
				((Gtk.Label)mnuNew.Child).Text = r.ReadLine();
				((Gtk.Label)mnuApriP.Child).Text = r.ReadLine();
				((Gtk.Label)mnuSalva.Child).Text = r.ReadLine();
				((Gtk.Label)mnuSalvaNome.Child).Text = r.ReadLine();
				((Gtk.Label)mnuChiudiP.Child).Text = r.ReadLine();
				((Gtk.Label)mnuCrea.Child).Text = r.ReadLine();
				((Gtk.Label)mnuApriG.Child).Text = r.ReadLine();
				((Gtk.Label)mnuAvvia.Child).Text = r.ReadLine();
				((Gtk.Label)mnuChiudiG.Child).Text = r.ReadLine();
				((Gtk.Label)mnuRecenti.Child).Text = r.ReadLine();
				((Gtk.Label)mnuPulisci.Child).Text = r.ReadLine();
				((Gtk.Label)mnuEsci.Child).Text = r.ReadLine();
				((Gtk.Label)mnuImpostazioni.Child).Text = r.ReadLine();
				((Gtk.Label)mnuPreferenze.Child).Text = r.ReadLine();
				((Gtk.Label)mnuAiuto.Child).Text = r.ReadLine();
				((Gtk.Label)mnuInformazioni.Child).Text = r.ReadLine();
				((Gtk.Label)mnuPromuovi.Child).Text = r.ReadLine();
				lblSchedaUno.Text = r.ReadLine();
				lblSchedaDue.Text = r.ReadLine();
				lblAdd.Text = r.ReadLine();
				lblAddDir.Text = r.ReadLine();
				lblElimina.Text = r.ReadLine();
				lblPulisci.Text = r.ReadLine();
				lblCrea.Text = r.ReadLine();
				tip_uno = r.ReadLine();
				tip_due = r.ReadLine();
				tip_tre = r.ReadLine();
				lblNome.Text = r.ReadLine();
				lblSecondi.Text = r.ReadLine();
				lblSuono.Text = r.ReadLine();
				lblExpander.Text = r.ReadLine();
				lblDimensioni.Text = r.ReadLine();
				optOriginali.Label = r.ReadLine();
				optJpg.Label = r.ReadLine();
				optPng.Label = r.ReadLine();
				optBilinear.Label = r.ReadLine();
				optHyper.Label = r.ReadLine();
				chkForce.Label = r.ReadLine();
				chkProporzioni.Label = r.ReadLine();
				lblCompressione.Text = r.ReadLine();
				cmdSfoglia.Label = r.ReadLine();
				
				r.Close();
				fs.Close();
			} catch (Exception e){
				Console.WriteLine(e);
			}
		}
	}

	public class wizardOne {

		[Widget] Gtk.Window Form2;
		[Widget] Gtk.Button cmdSfoglia;
		[Widget] Gtk.Button cmdNext;
		[Widget] Gtk.Entry txtNome;
		[Widget] Gtk.Entry txtPath;
		[Widget] Gtk.Image imgWizard;
		[Widget] Gtk.Label lblNome, lblPath;

		Gtk.Window Form1;
		Gtk.Notebook scheda;
		Gtk.FileChooserDialog chooser;
		Gtk.SpinButton spinSec;
		Gtk.Entry txtSuono, nome;
		Gtk.ListStore store;
		Gtk.MenuItem mnuNew;
		Gtk.ImageMenuItem mnuSalva, mnuSalvaNome, mnuChiudiP, mnuCrea;
		Gtk.Statusbar statusBar;
		Gtk.ProgressBar barWork;

		public wizardOne(Gtk.ListStore store, Gtk.Notebook scheda, Gtk.SpinButton spinSec, Gtk.Entry txtSuono, Gtk.Entry nome, Gtk.MenuItem mnuNew, Gtk.Statusbar statusBar, Gtk.ImageMenuItem mnuSalva, Gtk.ImageMenuItem mnuSalvaNome, Gtk.ImageMenuItem mnuChiudiP, Gtk.ImageMenuItem mnuCrea, Gtk.Window Form1, Gtk.ProgressBar barWork) {
			this.Form1 = Form1;
			this.store = store;
			this.scheda = scheda;
			this.spinSec = spinSec;
			this.txtSuono = txtSuono;
			this.nome = nome;
			this.mnuNew = mnuNew;
			this.mnuCrea = mnuCrea;
			this.mnuSalva = mnuSalva;
			this.mnuSalvaNome = mnuSalvaNome;
			this.mnuChiudiP = mnuChiudiP;
			this.statusBar = statusBar;
			this.barWork = barWork;
			
			Glade.XML glade = new Glade.XML("ssm.glade","Form2");
			glade.Autoconnect(this);

			if (ssm.unix_sys) imgWizard.FromFile = ssm.c_dir + "pixmaps/step1.gif";
			else imgWizard.FromFile = ssm.c_dir + "pixmaps\\step1.gif";
			
			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!ssm.unix_sys) icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps\\icon.png");
			else icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps/icon.png");
			Form2.GdkWindow.IconList = icons;
			
			// segnali
			cmdSfoglia.Clicked += cmdSfoglia_clicked;
			cmdNext.Clicked += cmdNext_clicked;

			load_language();
			
			Form2.ShowAll();
		}

		public void cmdSfoglia_clicked(object o, EventArgs args) {
			// apre la finestra di selezione files
			functions.create_chooser(ref chooser,functions.mex_text[35],Form2,Gtk.FileChooserAction.SelectFolder);
			chooser.SelectMultiple = false;
			
			Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
			chooser.Hide();

			// controlla la scelta
			if (risp == ResponseType.Ok) txtPath.Text = chooser.Filename;

			chooser.Destroy();
		}

		public void cmdNext_clicked(object o, EventArgs args) {
			// controlla che ci sia il nome della galleria
			if (txtNome.Text == "") {
				// messaggio di errore
				functions.message_box(Form2,functions.mex_text[20],MessageType.Warning);

				return;
			}

			ssm.nome_gallery = txtNome.Text;

			// memorizza il path se  corretto in una variabile e passa allo step numero 2
			if (txtPath.Text != "" && Directory.Exists(txtPath.Text)) ssm.path = txtPath.Text;
			
			// chiude la form2
			Form2.Destroy();
				
			new wizardTwo(store,scheda,spinSec,txtSuono,nome,mnuNew,statusBar,mnuSalva,mnuSalvaNome,mnuChiudiP,mnuCrea,Form1,barWork);
		}

		private void load_language() {
			try {
				FileStream fs;
				string path = ssm.c_dir + "lang/";

				if (ssm.p_language == "English") path += "en";
				else if (ssm.p_language == "Italiano") path += "it";
				else if (ssm.p_language == "Franais") path += "fr";
				else if (ssm.p_language == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);

				while (r.Peek() > -1 && r.ReadLine() != "***");

				lblNome.Text = r.ReadLine();
				lblPath.Text = r.ReadLine();
				cmdSfoglia.Label = r.ReadLine();

				r.Close();
				fs.Close();
			} catch {}
		}
	}

	public class wizardTwo {

		[Widget] Gtk.Window Form3;
		[Widget] Gtk.Button cmdSfoglia;
		[Widget] Gtk.Button cmdNext;
		[Widget] Gtk.Entry txtSuono;
		[Widget] Gtk.SpinButton spinSec;
		[Widget] Gtk.Image imgWizard;
		[Widget] Gtk.Label lblSecondi, lblSuono;

		Gtk.Window Form1;
		Gtk.ListStore store;
		Gtk.Notebook scheda;
		Gtk.FileChooserDialog chooser;
		Gtk.SpinButton spin;
		Gtk.Entry suono, nome;
		Gtk.MenuItem mnuNew;
		Gtk.ImageMenuItem mnuSalva, mnuSalvaNome, mnuChiudiP, mnuCrea;
		Gtk.Statusbar statusBar;
		Gtk.ProgressBar barWork;
		
		public wizardTwo(Gtk.ListStore store, Gtk.Notebook scheda, Gtk.SpinButton spin, Gtk.Entry suono, Gtk.Entry nome, Gtk.MenuItem mnuNew, Gtk.Statusbar statusBar, Gtk.ImageMenuItem mnuSalva, Gtk.ImageMenuItem mnuSalvaNome, Gtk.ImageMenuItem mnuChiudiP, Gtk.ImageMenuItem mnuCrea, Gtk.Window Form1, Gtk.ProgressBar barWork) {
			this.Form1 = Form1;
			this.store = store;
			this.scheda = scheda;
			this.spin = spin;
			this.suono = suono;
			this.nome = nome;
			this.mnuNew = mnuNew;
			this.mnuCrea = mnuCrea;
			this.mnuSalva = mnuSalva;
			this.mnuSalvaNome = mnuSalvaNome;
			this.mnuChiudiP = mnuChiudiP;
			this.statusBar = statusBar;
			this.barWork = barWork;
			
			Glade.XML glade = new Glade.XML("ssm.glade","Form3");
			glade.Autoconnect(this);
			
			if (ssm.unix_sys) imgWizard.FromFile = ssm.c_dir + "pixmaps/step2.gif";
			else imgWizard.FromFile = ssm.c_dir + "pixmaps\\step2.gif";

			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!ssm.unix_sys) icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps\\icon.png");
			else icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps/icon.png");
			Form3.GdkWindow.IconList = icons;
			
			// segnali
			cmdNext.Clicked += cmdNext_clicked;
			cmdSfoglia.Clicked += cmdSfoglia_clicked;

			load_language();

			Form3.ShowAll();
		}

		public void cmdSfoglia_clicked(object o, EventArgs args) {
			// apre la finestra di selezione files
			functions.create_chooser(ref chooser,functions.mex_text[39],Form3,Gtk.FileChooserAction.Open);
			chooser.SelectMultiple = false;

			// crea un filtro per i suoni
			string[] ext = {"*.mp3","*.wav","*.ogg"};
			Gtk.FileFilter filter = new Gtk.FileFilter();
			functions.create_filter(ref filter,ext,functions.mex_text[40]);
			chooser.AddFilter(filter);
			
			Gtk.ResponseType risp = (Gtk.ResponseType)chooser.Run();
			chooser.Hide();

			// controlla la scelta
			if (risp == ResponseType.Ok) txtSuono.Text = chooser.Filename;

			chooser.Destroy();
		}

		public void cmdNext_clicked(object o, EventArgs args) {
			// controlla il valore dello spin button
			if (spinSec.ValueAsInt < 1 || spinSec.ValueAsInt > 30) {
				// messaggio di errore
				functions.message_box(Form3,functions.mex_text[21],MessageType.Warning);

				return;
			}

			ssm.spin_sec = spinSec.ValueAsInt;

			// se  stato scelto un suono controlla la sua validit e lo memorizza in una variabile
			if (txtSuono.Text != "" && check_sound(txtSuono.Text)) ssm.suono = txtSuono.Text;

			// chiude la form3 e passa all'ultimo step
			Form3.Destroy();

			new wizardThree(store,scheda,spin,suono,nome,mnuNew,statusBar,mnuSalva,mnuSalvaNome,mnuChiudiP,mnuCrea,Form1,barWork);
		}

		// controlla se la stringa s  un suono valido (wav,mp3,ogg)
		private bool check_sound(string s) {
			if (File.Exists(s)) {
				string ext = s.Substring(s.Length-3);
				if (ext == "wav" || ext == "mp3" || ext == "ogg") return true;
			}

			return false;
		}
		
		private void load_language() {
			try {
				FileStream fs;
				string path = ssm.c_dir + "lang/";

				if (ssm.p_language == "English") path += "en";
				else if (ssm.p_language == "Italiano") path += "it";
				else if (ssm.p_language == "Franais") path += "fr";
				else if (ssm.p_language == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);

				for (int i = 0; i < 2; i++) while (r.Peek() > -1 && r.ReadLine() != "***");

				lblSecondi.Text = r.ReadLine();
				lblSuono.Text = r.ReadLine();
				cmdSfoglia.Label = r.ReadLine();

				r.Close();
				fs.Close();
			} catch {}
		}
	}

	public class wizardThree {

		[Widget] Gtk.Window Form4;
		[Widget] Gtk.Button cmdFine;
		[Widget] Gtk.Image imgWizard;
		[Widget] Gtk.Label lblFine, lblEnd;

		Gtk.Window Form1;
		Gtk.ListStore store;
		Gtk.Notebook scheda;
		Gtk.SpinButton spinSec;
		Gtk.Entry txtSuono, txtNome;
		Gtk.MenuItem mnuNew;
		Gtk.ImageMenuItem mnuSalva,mnuSalvaNome,mnuChiudiP,mnuCrea;
		Gtk.Statusbar statusBar;
		Gtk.ProgressBar barWork;

		System.Timers.Timer tmr = new System.Timers.Timer(200);
		ThreadNotify notify;

		public wizardThree(Gtk.ListStore store, Gtk.Notebook scheda, Gtk.SpinButton spinSec, Gtk.Entry txtSuono, Gtk.Entry txtNome, Gtk.MenuItem mnuNew, Gtk.Statusbar statusBar, Gtk.ImageMenuItem mnuSalva, Gtk.ImageMenuItem mnuSalvaNome, Gtk.ImageMenuItem mnuChiudiP, Gtk.ImageMenuItem mnuCrea, Gtk.Window Form1, Gtk.ProgressBar barWork) {
			this.Form1 = Form1;
			this.store = store;
			this.scheda = scheda;
			this.spinSec = spinSec;
			this.txtSuono = txtSuono;
			this.txtNome = txtNome;
			this.mnuNew = mnuNew;
			this.mnuCrea = mnuCrea;
			this.mnuSalva = mnuSalva;
			this.mnuSalvaNome = mnuSalvaNome;
			this.mnuChiudiP = mnuChiudiP;
			this.statusBar = statusBar;
			this.barWork = barWork;
			
			Glade.XML glade = new Glade.XML("ssm.glade","Form4");
			glade.Autoconnect(this);

			if (ssm.unix_sys) imgWizard.FromFile = ssm.c_dir + "pixmaps/step3.gif";
			else imgWizard.FromFile = ssm.c_dir + "pixmaps\\step3.gif";
			
			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!ssm.unix_sys) icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps\\icon.png");
			else icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps/icon.png");
			Form4.GdkWindow.IconList = icons;
			
			// segnali
			cmdFine.Clicked += cmdFine_clicked;
			tmr.Elapsed += new ElapsedEventHandler(tmr_elapsed);
			
			notify = new ThreadNotify(new ReadyEvent(move_progress));

			load_language();

			Form4.ShowAll();
		}

		public void cmdFine_clicked(object o, EventArgs args) {
			try {
				// termina la procedura guidata sistemando gli oggetti della finestra principale
				scheda.Sensitive = true;
				spinSec.Value = ssm.spin_sec;
				txtSuono.Text = ssm.suono;
				txtNome.Text = ssm.nome_gallery;
				Form1.Title = "Super Slide Me - " + ssm.nome_gallery;
				mnuNew.Sensitive = false;
				mnuCrea.Sensitive = true;
				mnuSalva.Sensitive = true;
				mnuSalvaNome.Sensitive = true;
				mnuChiudiP.Sensitive = true;

				// carica le immagini importate se ci sono
				if (Directory.Exists(ssm.path)) {
					// apre la finestra di attesa
					Form4.Hide();
					start_progress();
					functions.load_dir(Directory.GetFiles(ssm.path),store,null);
					stop_progress();
				}

				if (ssm.n_foto > 0) statusBar.Push(0,functions.mex_text[24] + ": " + ssm.n_foto + " - " + functions.mex_text[25] + ": " + functions.show_time(ref spinSec, ssm.n_foto));
				else statusBar.Push(0,functions.mex_text[24] + ": " + ssm.n_foto);
				Form4.Destroy();
			} catch {
				functions.message_box(Form4,functions.mex_text[22],MessageType.Error);
			}
		}

		private void start_progress() {
			tmr.Start();
		}

		private void stop_progress() {
			tmr.Stop();
			barWork.Fraction = 0;
		}

		public void tmr_elapsed(object o, ElapsedEventArgs args) {
			notify.WakeupMain();
		}

		private void move_progress() {
			barWork.Pulse();
		}
		
		private void load_language() {
			try {
				FileStream fs;
				string path = ssm.c_dir + "lang/";

				if (ssm.p_language == "English") path += "en";
				else if (ssm.p_language == "Italiano") path += "it";
				else if (ssm.p_language == "Franais") path += "fr";
				else if (ssm.p_language == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);

				for (int i = 0; i < 3; i++) while (r.Peek() > -1 && r.ReadLine() != "***");

				lblFine.Text = r.ReadLine();
				lblEnd.Text = r.ReadLine();

				r.Close();
				fs.Close();
			} catch {}
		}
	}

	public class visualizzatore {

		[Widget] Gtk.Window Form5;
		[Widget] Gtk.Image imgFoto;

		// ridimensiona l'immagine (800x600~)
		private void adjust_size(ref Gdk.Pixbuf pix) {
			try {
				int h;
				int w = 0;
			
				// prima ridimensiona fissando la larghezza se necessario
				if (pix.Width > 800 || pix.Height > 600) {
					// ricalcola l'altezza
					h = (pix.Height * 800) / pix.Width;

					// se l'altezza non entra nel range 600 ridimensiona in base all'altezza
					if (h > 600) w = (pix.Width * 600) / pix.Height;

					// scala l'immagine
					if (w == 0) pix = pix.ScaleSimple(800,h,Gdk.InterpType.Bilinear);
					else pix = pix.ScaleSimple(w,600,Gdk.InterpType.Bilinear);
				}
			} catch {}
		}

		public visualizzatore(string f, string r) {
			Glade.XML glade = new Glade.XML("ssm.glade","Form5");
			glade.Autoconnect(this);

			Form5.ShowAll();
			Form5.Maximize();

			Gdk.Pixbuf pix = new Gdk.Pixbuf(f);

			// controlla se bisogna girare l'immagine
			if (r == "l") functions.rotate_left(ref pix);
			else if (r == "r") functions.rotate_right(ref pix);
			else if (r == "d") functions.rotate_down(ref pix);
			
			adjust_size(ref pix);

			imgFoto.FromPixbuf = pix;
		}
	}

	public class preferenze {
		
		[Widget] Gtk.Window Form6;
		[Widget] Gtk.CheckButton chkElimina, chkPulisci, chkBig, chkGalleria;
		[Widget] Gtk.Button cmdSalva;
		[Widget] Gtk.ComboBox cboLinguaggio;
		[Widget] Gtk.Label lblLingua, lblSalva;

		public preferenze() {
			Glade.XML glade = new Glade.XML("ssm.glade","Form6");
			glade.Autoconnect(this);

			cmdSalva.Clicked += cmdSalva_clicked;
			cboLinguaggio.Changed += cboLinguaggio_change;

			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!ssm.unix_sys) icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps\\preferences.png");
			else icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps/preferences.png");
			Form6.GdkWindow.IconList = icons;

			load_language(ssm.p_language);
			load_options();
			
			Form6.ShowAll();
		}

		// salva le opzioni nelle variabili statiche
		public void cmdSalva_clicked(object o, EventArgs args) {
			ssm.conf_elimina = chkElimina.Active;
			ssm.conf_pulisci = chkPulisci.Active;
			ssm.apri_big = chkBig.Active;
			ssm.apri_galleria = chkGalleria.Active;

			if (ssm.p_language != cboLinguaggio.ActiveText) {
				ssm.p_language = cboLinguaggio.ActiveText;

				// messaggio
				functions.message_box(Form6,functions.mex_text[23],MessageType.Info);
			}

			// chiude la finestra
			Form6.Destroy();
		}

		// cambio della lingua
		public void cboLinguaggio_change(object o, EventArgs args) {
			load_language(cboLinguaggio.ActiveText);
		}

		// carica le opzioni dalle variabili
		private void load_options() {
			chkElimina.Active = ssm.conf_elimina;
			chkPulisci.Active = ssm.conf_pulisci;
			chkBig.Active = ssm.apri_big;
			chkGalleria.Active = ssm.apri_galleria;
			
			TreeIter iter;
			string p;
			if (ssm.p_language == "English") p = "0";
			else if (ssm.p_language == "Italiano") p = "1";
			else if (ssm.p_language == "Franais") p = "2";
			else p = "3";
			cboLinguaggio.Model.GetIterFromString(out iter, p);
			cboLinguaggio.SetActiveIter(iter);
		}
		
		private void load_language(string lang) {
			try {
				FileStream fs;
				string path = ssm.c_dir + "lang/";

				if (lang == "English") path += "en";
				else if (lang == "Italiano") path += "it";
				else if (lang == "Franais") path += "fr";
				else if (lang == "Espanol") path += "es";

				fs = new FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read);

				StreamReader r = new StreamReader(fs);

				for (int i = 0; i < 4; i++) while (r.Peek() > -1 && r.ReadLine() != "***");

				chkElimina.Label = r.ReadLine();
				chkPulisci.Label = r.ReadLine();
				chkBig.Label = r.ReadLine();
				chkGalleria.Label = r.ReadLine();
				lblLingua.Text = r.ReadLine();
				lblSalva.Text = r.ReadLine();

				r.Close();
				fs.Close();
			} catch {}
		}
	}

	// la presentazione della galleria
	public class presentazione {

		[Widget] Gtk.Window Form8;
		[Widget] Gtk.Image imgFoto;
		[Widget] Gtk.Label lblCont;
		[Widget] Gtk.HBox menu;
		[Widget] Gtk.Button cmdPlayPause, cmdPrev, cmdNext, cmdEsci;

		Gtk.Window finestra;
		Gtk.HBox hPlayPause, hPrev, hNext, hEsci;
		Gtk.Image imgPlayPause, imgPrev, imgNext, imgEsci;

		System.Timers.Timer tmr = new System.Timers.Timer(ssm.spin_sec * 1000);
		System.Timers.Timer tmr_titolo = new System.Timers.Timer(5000);
		System.Timers.Timer tmr_mouse = new System.Timers.Timer(5000);
      ThreadNotify notify, notify_titolo, notify_mouse;
		Music mus = Mixer.Music;
				
		bool activated = false, mouse_visible = true;
		int corr = 0, screen_width = 0, screen_height = 0;
		string[] immagine;
		string path, suono = "";
		
		public presentazione(string path) {
			try {
				Glade.XML glade = new Glade.XML("ssm.glade","Form8");
				glade.Autoconnect(this);

				this.path = path;

				Form8.MapEvent += Form8_activate;
				Form8.KeyPressEvent += Form8_keypress;
				Form8.MotionNotifyEvent += Form8_mousemove;
				notify = new ThreadNotify(new ReadyEvent(view_image));
				notify_titolo = new ThreadNotify(new ReadyEvent(view_title));
				notify_mouse = new ThreadNotify(new ReadyEvent(hide_mouse));
         	tmr.Elapsed += new ElapsedEventHandler(tmr_elapsed);
				tmr_titolo.Elapsed += new ElapsedEventHandler(tmr_titolo_elapsed);
				tmr_mouse.Elapsed += new ElapsedEventHandler(tmr_mouse_elapsed);
				cmdPlayPause.Clicked += cmdPlayPause_clicked;
				cmdPrev.Clicked += cmdPrev_clicked;
				cmdNext.Clicked += cmdNext_clicked;
				cmdEsci.Clicked += cmdEsci_clicked;

				// ricava la risoluzione dello schermo
				Gdk.Screen screen = Gdk.Screen.Default;
				screen_width = screen.Width;
				screen_height = screen.Height;

				Form8.Fullscreen();
				Form8.ShowAll();
			} catch {}
		}

		public void Form8_mousemove(object o, MotionNotifyEventArgs args) {			
			try {
				show_menu();
			} catch {}
		}

		public void Form8_activate(object o, MapEventArgs args) {
			try {
				if (activated) return;
				activated = true;

				// carica tutte le immagini in un vettore
				immagine = Directory.GetFiles(path);

				// se c' il suono lo toglie dal vettore e lo memorizza in una variabile
				if (immagine[immagine.Length-1].EndsWith("mp3") || immagine[immagine.Length-1].EndsWith("ogg") || immagine[immagine.Length-1].EndsWith("wav")) {
					suono = immagine[immagine.Length-1];
					immagine[immagine.Length-1] = "";
				}

				if (suono != "") mus.Load(suono);

				// crea una finestra per visualizzare il nome della galleria
				finestra = new Gtk.Window("");
				finestra.TransientFor = Form8;
				finestra.DestroyWithParent = true;

				Gtk.Label lblTitolo = new Gtk.Label(ssm.nome_gallery);
				Pango.FontDescription fd = new Pango.FontDescription();
				fd.Weight = Pango.Weight.Heavy;
				fd.Size = screen_width * 45;
				lblTitolo.ModifyFont(fd);
				lblTitolo.ModifyFg(Gtk.StateType.Normal, new Gdk.Color(255,255,255));
				finestra.Add(lblTitolo);
				finestra.Fullscreen();
				Form8.ModifyBg(Gtk.StateType.Normal, new Gdk.Color(0,0,0));
				finestra.ModifyBg(Gtk.StateType.Normal, new Gdk.Color(0,0,0));
				lblCont.ModifyFg(Gtk.StateType.Normal, new Gdk.Color(255,255,255));

				notify.WakeupMain();
			
				finestra.ShowAll();
				tmr_titolo.Start();

				// setta il cursore invisibile e sistema il menu
				finestra.GdkWindow.Cursor = functions.ghost_cursor();
				Form8.GdkWindow.Cursor = functions.ghost_cursor();
				mouse_visible = false;
				menu.Visible = false;

				hPlayPause = new HBox(false,0);
				imgPlayPause = new Image(ssm.c_dir + "pixmaps/pause.png");
				hPlayPause.PackStart(imgPlayPause,false,false,0);
				imgPlayPause.Show();
				hPlayPause.Show();
				cmdPlayPause.Add(hPlayPause);
				
				hPrev = new HBox(false,0);
				imgPrev = new Image(ssm.c_dir + "pixmaps/prev.png");
				hPrev.PackStart(imgPrev,false,false,0);
				imgPrev.Show();
				hPrev.Show();
				cmdPrev.Add(hPrev);
				
				hNext = new HBox(false,0);
				imgNext = new Image(ssm.c_dir + "pixmaps/next.png");
				hNext.PackStart(imgNext,false,false,0);
				imgNext.Show();
				hNext.Show();
				cmdNext.Add(hNext);

				hEsci = new HBox(false,0);
				imgEsci = new Image(ssm.c_dir + "pixmaps/exit.png");
				hEsci.PackStart(imgEsci,false,false,0);
				imgEsci.Show();
				hEsci.Show();
				cmdEsci.Add(hEsci);
			} catch (Exception e) {
				functions.message_box(Form8,functions.mex_text[11],MessageType.Error);
				Console.WriteLine(e);
			}
		}

		// pressione del tasto esc
		public void Form8_keypress(object o, KeyPressEventArgs args) {
			try {
				if (args.Event.KeyValue == 65307) {
					mus.Stop();
					tmr.Stop();
					if (tmr_titolo.Enabled) tmr_titolo.Stop();
					if (tmr_mouse.Enabled) tmr_mouse.Stop();
					Form8.Destroy();
				}
			} catch {}
		}

		public void cmdPlayPause_clicked(object o, EventArgs args) {
			try {
				if (tmr.Enabled) {
					tmr.Stop();
					imgPlayPause.FromFile = ssm.c_dir + "pixmaps/play.png";
				} else {
					tmr.Start();
					imgPlayPause.FromFile = ssm.c_dir + "pixmaps/pause.png";
				}

				tmr_mouse.Stop();
				tmr_mouse.Start();
			} catch {}
		}

		// visualizza la foto precedente
		public void cmdPrev_clicked(object o, EventArgs args) {
			try {
				if (tmr.Enabled) cmdPlayPause.Click();

				if (corr < 2) return;
			
				corr -= 2;
				view_image();
			
				tmr_mouse.Stop();
				tmr_mouse.Start();
			} catch {}
		}

		// visualizza la foto successiva
		public void cmdNext_clicked(object o, EventArgs args) {
			try {
				if (tmr.Enabled) cmdPlayPause.Click();

				if (corr == immagine.Length || immagine[corr] == "") return;

				view_image();
			
				tmr_mouse.Stop();
				tmr_mouse.Start();
			} catch {}
		}

		public void cmdEsci_clicked(object o, EventArgs args) {
			try {
				mus.Stop();
				tmr.Stop();
				if (tmr_titolo.Enabled) tmr_titolo.Stop();
				if (tmr_mouse.Enabled) tmr_mouse.Stop();
				Form8.Destroy();
			} catch {}
		}

		// elapsed del timer
		public void tmr_elapsed(object o, ElapsedEventArgs args) {
			try {
				notify.WakeupMain();
			} catch {}
		}

		public void tmr_titolo_elapsed(object o, ElapsedEventArgs args) {
			try {
				notify_titolo.WakeupMain();
			} catch {}
		}

		public void tmr_mouse_elapsed(object o, ElapsedEventArgs args) {
			try {
				notify_mouse.WakeupMain();
			} catch {}
		}

		// visualizza la prossima immagine presente nella galleria
		public void view_image() {
			try {
				if (!mus.IsPlaying() && suono != "") mus.Play(-1);
				if (corr == immagine.Length || immagine[corr] == "") {
					mus.Stop();
					tmr.Stop();
					if (tmr_mouse.Enabled) tmr_mouse.Stop();
					Form8.Destroy();

					return;
				}
			
				Gdk.Pixbuf pix = new Gdk.Pixbuf(immagine[corr]);

				// se l'immagine  pi grande della risoluzione dello schermo la rimpicciolisce controllando il flag proporzioni
				// se l'immagine  pi piccola della risoluzione dello schermo la ingrandisce forzandola
				if (pix.Width != screen_width || pix.Height != screen_height) {
					if (ssm.filtro == 0) functions.size_preview_large(ref pix,screen_width + 250,screen_height + 150,true,ssm.propor,Gdk.InterpType.Bilinear);
					else functions.size_preview_large(ref pix,screen_width + 250,screen_height + 150,true,ssm.propor,Gdk.InterpType.Hyper);
				}
				imgFoto.FromPixbuf = pix;
				
				corr++;

				lblCont.Text = corr + " di " + ssm.n_foto;

				if (corr == immagine.Length || immagine[corr] == "") cmdNext.Sensitive = false;
				else cmdNext.Sensitive = true;

				if (corr == 1) cmdPrev.Sensitive = false;
				else cmdPrev.Sensitive = true;
			} catch {}
		}

		public void view_title() {
			try {
				tmr_titolo.Stop();
				tmr.Start();
				finestra.Destroy();
			} catch {}
		}

		public void hide_mouse() {
			try {
				tmr_mouse.Stop();
				Form8.GdkWindow.Cursor = functions.ghost_cursor();
				mouse_visible = false;
				menu.Visible = false;
			} catch {}
		}

		private void show_menu() {
			try {
				if (!mouse_visible) {
					Form8.GdkWindow.Cursor = null;
					mouse_visible = true;
					menu.Visible = true;
					tmr_mouse.Start();
				}
			} catch {}
		}
	}

	public class informazioni {
		
		[Widget] Gtk.Window Form9;

		public informazioni() {
			Glade.XML glade = new Glade.XML("ssm.glade","Form9");
			glade.Autoconnect(this);
			
			Gdk.Pixbuf[] icons = new Gdk.Pixbuf[1];
			if (!ssm.unix_sys) icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps\\information.png");
			else icons[0] = new Gdk.Pixbuf(ssm.c_dir + "pixmaps/information.png");
			Form9.GdkWindow.IconList = icons;

			Form9.ShowAll();
		}

	}
}
